/*
  NSPasteboard-NIRAdditions.m

  HebrewService - system services for Hebrew users.
 
  Copyright (C) 2003-2006 Nir Soffer
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details:
  http://www.gnu.org/licenses/gpl.html
*/

#import "NSPasteboard-NIRAdditions.h"
#import "NSAttributedString-NIRAdditions.h"


@implementation NSPasteboard (NSPasteboardNIRAdditions)

//
// Put data
//

- (BOOL)setObject:(id)object forType:(NSString *)type; {
    /* Put Specific type on the pasteboard

    Based on existing setMethods or specific objects pasteboard mehtods, like
    NSColor -writeToPasteboard. Instead of reading the documentaion each time
    just call this method and it will use the right method for the type
    */

    BOOL success = NO;
    
    if ([type isEqualToString:NSColorPboardType] ||
        [type isEqualToString:NSURLPboardType]) {
        [object writeToPasteboard:self];
        // writeToPasteboard does not return anything, so return YES
        success = YES;
    } else if ([type isEqualToString:NSFileContentsPboardType]) {
        success = [self writeFileContents:object];
    } else if ([type isEqualToString:NSFilenamesPboardType]) {
        success = [self setPropertyList:object forType:type];
    } else if ([type isEqualToString:NSStringPboardType]) {
        success = [self setString:object forType:type];
    } else if ([object isKindOfClass:[NSData class]]) {
        // other types are writen as data, but only if they are
        success = [self setData:object forType:type];
    }
    return success;
}


//
// Shortcuts - declare and set object in one call
//

- (BOOL)putObject:(id)object forType:(NSString *)type; {
    /* Declare the type with nil owner, then invoke -setObject:forType: */

    NSArray *types = [[NSArray alloc] initWithObjects:type, nil];
    [self declareTypes:types owner:nil];
    [types release];
    return [self setObject:object forType:type];
}

- (BOOL)putObjects:(NSDictionary *)objects; {
    /* Shortcut for puting pile of objects at once

     Declare objects keys with nil owner, then invoke -setObect:forType:
     for each
    */

    NSString *type;
    NSEnumerator *typeEnumerator;
    id object;

    [self declareTypes:[objects allKeys] owner:nil];
    typeEnumerator = [objects keyEnumerator];
    while ((type = [typeEnumerator nextObject])) {
        object = [objects objectForKey:type];
        // Add object or stop and return NO
        if ([self setObject:object forType:type] == NO) return NO;
    }
    return YES;
}

- (BOOL)putString:(NSString *)string; {
    /* Shortcut for puting strings for NSStringPboardType. */
    return [self putObject:string forType:NSStringPboardType];
}

- (BOOL)putAttributedString:(NSAttributedString *)attString
                    forType:(NSString *)type; {
    /* Put the best and most eficient representation for type */

    NSArray *types = [[NSArray alloc] initWithObjects:type, nil];
    [self declareTypes:types owner:nil];
    [types release];

    id object = nil;
    if ([type isEqualToString:NSStringPboardType]) {
        object = [attString string];
    } else if ([type isEqualToString:NSRTFPboardType]) {
        object = [attString RTF];
    } else if ([type isEqualToString:NSRTFDPboardType]) {
        object = [attString RTFD];
    }
    return [self setObject:object forType:type];
}


//
// Get data
//

- (id)objectForType:(NSString *)type; {
    // The reverse of setObject:forType:

    id object = nil;
    if ([type isEqualToString:NSColorPboardType]) {
        object = [self readColor];
    } else if ([type isEqualToString:NSURLPboardType]) {
        object = [self readURL];
    } else if ([type isEqualToString:NSFilenamesPboardType]) {
        // Return Array of NStrings
        object = [self propertyListForType:type];
    } else if ([type isEqualToString:NSStringPboardType]) {
        object = [self stringForType:type];
    } else {
        // other types are read with -dataForType:
        object = [self dataForType:type];
    }
    return object;
}

- (NSColor *)readColor; {
    /* Return color object if one exists or nil */
    return [NSColor colorFromPasteboard:self];
}

- (NSURL *)readURL; {
    /* Return url object if one exists or nil */
    return [NSURL URLFromPasteboard:self];
}

- (NSMutableAttributedString *)attributedStringForType:(NSString *)type; {
    /* Return the best text representation available

    The pasteboard could hold a string, RTF or RTFD data. We read them into
    a mutable attributed string, which can handle both plain and rich text.
    */

    NSString* string;
    NSData *data;
    NSMutableAttributedString *result = nil;

    if ([type isEqualToString:NSStringPboardType]) {
        string = [self stringForType:type];
        result = [[NSMutableAttributedString alloc] initWithString:string];
    } else if ([type isEqualToString:NSRTFDPboardType]) {
        data = [self dataForType:type];
        result = [[NSMutableAttributedString alloc] initWithRTFD:data
                                              documentAttributes:NULL];
    } else if ([type isEqualToString:NSRTFPboardType]) {
        data = [self dataForType:type];
        result = [[NSMutableAttributedString alloc] initWithRTF:data
                                             documentAttributes:NULL];
    }
    return [result autorelease];
}

@end
