/*
  NSString-FriBidiAdditions.h
 
  HebrewService - system services for Hebrew users.
 
  Copyright (C) 2003-2006 Nir Soffer
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details:
  http://www.gnu.org/licenses/gpl.html

  TODO
   
  Consider creating a conventor class that allocates one buffer during
  one string convertion. The buffer will be created with sane initial
  capacity, e.g (80-100 characters?) and will allocate more memory as
  needed. -orderVisually will create an instace of the conventor per
  call.
*/


#import "NSString-FriBidiAdditions.h"
#import "fribidi.h"

static NSString * orderLineVisually(NSString *line, FriBidiCharType direction)
{
    NSString *result = nil; // failure by default
    unsigned int stringLength = [line length];
    
    // Bytes buffers hold the string using either UCS4 or UTF8 encoding
    // both up to 4 bytes per character.
    // TODO: consider reusing a static buffer instead of allocating and 
    // freeing on each call.
    
    size_t bufferSize = (stringLength + 1) * 4;
    char *a = NULL, *b = NULL;
    
    a = malloc(bufferSize);
    if (a == NULL)
        goto cleanup;
    
    b = malloc(bufferSize);
    if (b == NULL)
        goto cleanup;
    
    // Encode, reorder and decode back. The functions here get two buffers 
    // and copy from one buffer to the other. Each call use the ouput buffer 
    // of the previous call, which save extra memory allocation.
    // a -> b, b -> a, a -> b...
    
    if (! CFStringGetCString((CFStringRef)line, a, bufferSize,
                             kCFStringEncodingUTF8))
        goto cleanup;
    
    fribidi_utf8_to_unicode(a, strlen(a), (FriBidiChar *)b);
    
    if (! fribidi_log2vis((FriBidiChar *)b, stringLength, &direction, 
                          (FriBidiChar *)a, NULL, NULL, NULL))
        goto cleanup;
    
    fribidi_unicode_to_utf8((FriBidiChar *)a, stringLength, b);
    
    result = (NSString *)CFStringCreateWithCString(NULL, b, 
                                                   kCFStringEncodingUTF8);
    [result autorelease];
    
cleanup:
    free(a);
    free(b);
    
    return result;
}


@implementation NSString (FriBidiAdditions)

// Iterate line content and order each visually
- (NSString *)stringByOrderingVisually;
{
    unsigned maxIndex = [self length];
    unsigned lineStart = 0, lineEnd = 0, contentsEnd = 0;
    NSMutableString *result = [[self mutableCopy] autorelease];    
    
    while (lineEnd < maxIndex) {        
        [self getLineStart:&lineStart 
                       end:&lineEnd 
               contentsEnd:&contentsEnd 
                  forRange:NSMakeRange(lineEnd, 0)];
        
        NSRange lineRange = NSMakeRange(lineStart, contentsEnd - lineStart);
        if (lineRange.length == 0)
            continue;
        
        NSString *line = [self substringWithRange:lineRange];
        [result replaceCharactersInRange:lineRange withString:
            orderLineVisually(line, FRIBIDI_TYPE_RTL)];
    }
    return result;
}

@end
