/*

NSPasteboard-NIRAdditionsTest.m

DirectionService - controll writting direction
Copyright (C) 2003 Nir Soffer

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details: http://www.gnu.org/licenses/gpl.html

*/

#import "NSPasteboard-NIRAdditionsTest.h"
#import "NSPasteboard-NIRAdditions.h"
#import "NSString-NIRAdditions.h"
#import "NSAttributedString-NIRAdditions.h"

static NSString *NIRUserDefinedPboardType = @"User Defined Pasteboard Type";

@implementation NSPasteboardNIRAdditionsTest

- (void)setUp
{
    pboard = [NSPasteboard pasteboardWithUniqueName];
}

- (void)tearDown
{
}

- (void)testSetObjectForTypeColor {
    // Put color on the pasteboard
    [self testObjects];
    NSString *type = NSColorPboardType;
    [pboard declareTypes:[NSArray arrayWithObject:type] owner:nil];
    [self assertTrue:[pboard setObject:color forType:type]];
    [self assertTrue:[[pboard types] containsObject:type]];
    [self assert:color equals:[pboard readColor]];
}

- (void)testSetObjectForTypeString {
    // Put string on the pasteboard
    [self testObjects];
    NSString *type = NSStringPboardType;
    [pboard declareTypes:[NSArray arrayWithObject:type] owner:nil];
    [self assertTrue:[pboard setObject:string forType:type]];
    [self assertTrue:[[pboard types] containsObject:type]];
    [self assert:string equals:[pboard stringForType:type]];
}

- (void)testSetObjectForTypeRTF {
    // Put RTF on the pasteboard
    [self testObjects];
    NSString *type = NSRTFPboardType;
    NSData *data = [attString RTF];
    [pboard declareTypes:[NSArray arrayWithObject:type] owner:nil];
    [self assertTrue:[pboard setObject:data forType:type]];
    [self assertTrue:[[pboard types] containsObject:type]];
    [self assert:data equals:[pboard dataForType:type]];
}

- (void)testSetObjectForTypeRTFD {
    // Put RTFD on the pasteboard
    [self testObjects];
    NSString *type = NSRTFDPboardType;
    NSData *data = [attStringWithAttachment RTFD];
    [pboard declareTypes:[NSArray arrayWithObject:type] owner:nil];
    [self assertTrue:[pboard setObject:data forType:type]];
    [self assertTrue:[[pboard types] containsObject:type]];
    [self assert:data equals:[pboard dataForType:type]];
}

- (void)testSetObjectForTypeUserDefinedType {
    // Put data on the pasteboard
    [self testObjects];
    NSString *type = NIRUserDefinedPboardType;
    NSData *data = [attString RTF];
    [pboard declareTypes:[NSArray arrayWithObject:type] owner:nil];
    [self assertTrue:[pboard setObject:data forType:type]];
    [self assertTrue:[[pboard types] containsObject:type]];
    [self assert:data equals:[pboard dataForType:type]];
}

- (void)testSetObjectForTypeUserDefinedTypeNonData {
    // Try to put user defined type with an object which is not NSData
    NSString *type = NIRUserDefinedPboardType;
    [pboard declareTypes:[NSArray arrayWithObject:type] owner:nil];
    [self assertFalse:[pboard setObject:@"This is not NSData" forType:type]];
}

- (void)testPutObjectForType {
    // Since this invoke -setObject:forType: which is well tested,  we check
    // only one type to see if the declare thing works
    [self testObjects];
    NSString *type = NSStringPboardType;
    [self assertTrue:[pboard putObject:string forType:type]];
    [self assertTrue:[[pboard types] containsObject:type]];
    [self assert:string equals:[pboard stringForType:type]];
}

- (void)testPutObjects {
    // Put objects from dictionary and get all of them back
    NSString *type;
    NSDictionary *testObjects = [self testObjects];
    [pboard putObjects:testObjects];
    NSArray *types = [pboard types];
    NSEnumerator *typeEnumerator = [testObjects keyEnumerator];
    while ((type = [typeEnumerator nextObject])) {
        [self assertTrue:[types containsObject:type]];
        [self assert:[testObjects objectForKey:type]
              equals:[pboard objectForType:type]];
    }
}

- (void)testPutObjectsWithNonData {
    // Try to put user defined type with an object which is not NSData
    NSString *type = NIRUserDefinedPboardType;
    [pboard declareTypes:[NSArray arrayWithObject:type] owner:nil];
    NSDictionary *objects = [NSDictionary dictionaryWithObjectsAndKeys:
        @"This is not NSData", NIRUserDefinedPboardType, nil];
    [self assertFalse:[pboard putObjects:objects]];
}

- (void)testAttributedStringForTypeString {
    // The pasteboard should return the string

    [pboard putObjects:[self testObjects]];
    NSString *type = [pboard availableTypeFromArray:
        [NSArray arrayWithObject:NSStringPboardType]];
    [self assert:[[pboard attributedStringForType:type] string]
          equals:string
         message:@"string failed"];
}

- (void)testAttributedStringForTypeRTF {
    // The pasteboard should return the attributed string

    [pboard putObjects:[self testObjects]];
    NSString *type = [pboard availableTypeFromArray:[NSArray arrayWithObjects:
        NSRTFPboardType, NSStringPboardType, nil]];
    NSAttributedString *returned = [pboard attributedStringForType:type];
    [self assertTrue:[returned isEqualToAttributedString:attString]
             message:@"attributed string failed"];
}

- (void)testAttributedStringForTypeRTFD {
    // The pasteboard should return the attributed string with attachment

    [pboard putObjects:[self testObjects]];
    NSString *type = [pboard availableTypeFromArray:[NSArray arrayWithObjects:
        NSRTFDPboardType, NSRTFPboardType, NSStringPboardType, nil]];
    NSAttributedString *returned = [pboard attributedStringForType:type];

    // Test only the string, because attributes don't match when adding
    // attachments. Probaablly bug in Cocoa.
    [self assert:[returned string]
          equals:[attStringWithAttachment string]
         message:@"attributed string with attachment strings don't match"];
}

- (void)testPutAttributedStringForTypeString {
    // Put only the string for NSStringPboardType
    [self testObjects];
    NSString *type = NSStringPboardType;
    [self assertTrue:[pboard putAttributedString:attString forType:type]];
    [self assert:[pboard stringForType:type] equals:[attString string]];
}

- (void)testPutAttributedStringForTypeRTF {
    // Put RTF Data for NSStringPboardType
    [self testObjects];
    NSString *type = NSRTFPboardType;
    [self assertTrue:[pboard putAttributedString:attString forType:type]];
    [self assert:[pboard attributedStringForType:type] equals:attString];
}

- (void)testPutAttributedStringForTypeRTFD {
    // Put RTFD Data for NSStringPboardType
    [self testObjects];
    NSString *type = NSRTFDPboardType;
    [self assertTrue:[pboard putAttributedString:attStringWithAttachment
                                         forType:type]];

    // Test only the string, because attributes don't match when adding
    // attachments. Probaablly bug in Cocoa.
    [self assert:[[pboard attributedStringForType:type] string]
          equals:[attStringWithAttachment string]];
}

// Utilities

- (NSAttributedString *)attributedStringFromString:(NSString *)aString
                                        attachment:(NSString *)attachment; {
    // Return an attributed aString from aString with optional file attachment

    // Create attributes
    NSDictionary *attributes = [[NSDictionary alloc] initWithObjectsAndKeys:
        [NSFont systemFontOfSize:12],
        NSFontAttributeName,
        [NSColor redColor],
        NSForegroundColorAttributeName,
        [NSParagraphStyle defaultParagraphStyle],
        NSParagraphStyleAttributeName, nil];

    // Create attributed aString
    NSMutableAttributedString *attributedString =
        [[NSMutableAttributedString alloc] initWithString:aString
                                               attributes:attributes];
    [attributes release];
    
    // Add optional attachment
    if (attachment) {
        NSFileWrapper *file = [[NSFileWrapper alloc] initWithPath:attachment];
        NSTextAttachment *attachment =
            [[NSTextAttachment alloc] initWithFileWrapper:file];
        [file release];
        NSAttributedString *attachmentString =
            [NSAttributedString attributedStringWithAttachment:attachment];
        [attachment release];
        [attributedString insertAttributedString:attachmentString
                                         atIndex:[attributedString length]];
        [attributedString fixAttributesInRange:[attributedString range]];
    }

    return [attributedString autorelease];
}

- (NSDictionary *)testObjects; {
    // Create test objects return them in a dictionary ready for Pasteboard
    // -putObjects:owner:
    
    string = @"Test String";
    attString = [self attributedStringFromString:@"Test RTF String"
                                      attachment:nil];
    attStringWithAttachment =
        [self attributedStringFromString:@"Test RTFD String"
                              attachment:@"../Test Files/image.png"];
    userDefinedData = [attStringWithAttachment RTFD];
    color = [NSColor redColor];
    NSDictionary *objects = [NSDictionary dictionaryWithObjectsAndKeys:
        string, NSStringPboardType,
        [attString RTF], NSRTFPboardType,
        [attStringWithAttachment RTFD], NSRTFDPboardType,
        color, NSColorPboardType,
        userDefinedData, NIRUserDefinedPboardType, nil];

    return objects;
}
@end
